package com.loafle.overflow.container.service;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.registry.RPCRegistry;
import com.loafle.overflow.container.Container;
import com.loafle.overflow.core.annotation.ProbeAPI;
import com.loafle.overflow.core.annotation.RPCService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.AnnotationUtils;

/**
 * Service
 */
@Configuration
public class Service implements InitializingBean, ApplicationContextAware {
  private static final Logger logger = LoggerFactory.getLogger(Service.class);

  private ApplicationContext applicationContext;

  @Autowired
  @Qualifier(Container.RPC_REGISTRY)
  private RPCRegistry rpcRegistry;

  @Autowired
  @Qualifier(Container.SERVICES)
  private Map<String, Object> services;

  @Override
  public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
    this.applicationContext = applicationContext;
  }

  @Bean(Container.SERVICES)
  public Map<String, Object> services() {
    return this.applicationContext.getBeansWithAnnotation(RPCService.class);
  }

  @Override
  public void afterPropertiesSet() throws Exception {
    if (null == this.services || 0 == this.services.size()) {
      logger.debug("there is not service");
      return;
    }

    this.services.forEach((name, bean) -> {
      this.registerRPCService(bean, name);
    });
  }

  private void registerRPCService(Object bean, String name) {
    try {
      Class<?> clazz = bean.getClass();
      Method[] methods = clazz.getMethods();

      Map<String, Method> methodMap = new HashMap<>();

      for (Method method : methods) {
        String methodName = this.getRPCMethodName(method);
        if (null == methodName) {
          continue;
        }
        methodMap.put(methodName, method);
      }

      if (0 == methodMap.size()) {
        return;
      }

      this.rpcRegistry.registerService(bean, name, methodMap);
    } catch (RPCException e) {
      logger.error("RPCRegistry", e);
    }
  }

  private String getRPCMethodName(Method method) {
    ProbeAPI a = AnnotationUtils.findAnnotation(method, ProbeAPI.class);
    if (null == a) {
      return null;
    }
    String name = a.value();
    if (name.isEmpty()) {
      name = method.getName();
    }
    return name;
  }
}